@@ -25,6 +25,18 @@ population = [
      'Vertical axis buffer': 11, 'Horizontal axis buffer': 11, 'Vertical end buffer': 11, 'Horizontal end buffer': 20,
      'Default name': 'Rxx', 'Optional': False},
 
+    {'Connector location': 'transition', 'Diagram type': 'state machine', 'Notation': 'xUML',
+     'Vertical axis buffer': 6, 'Horizontal axis buffer': 6, 'Vertical end buffer': 11, 'Horizontal end buffer': 20,
+     'Default name': 'Event XX', 'Optional': False},
+
+    {'Connector location': 'initial transition', 'Diagram type': 'state machine', 'Notation': 'xUML',
+     'Vertical axis buffer': 6, 'Horizontal axis buffer': 6, 'Vertical end buffer': 11, 'Horizontal end buffer': 20,
+     'Default name': 'Event XX', 'Optional': False},
+
+    {'Connector location': 'deletion transition', 'Diagram type': 'state machine', 'Notation': 'xUML',
+     'Vertical axis buffer': 6, 'Horizontal axis buffer': 6, 'Vertical end buffer': 11, 'Horizontal end buffer': 20,
+     'Default name': 'Event XX', 'Optional': False},
+
     # Connector location names
     {'Connector location': 'class mult', 'Diagram type': 'class', 'Notation': 'Starr',
      'Vertical axis buffer': 8, 'Horizontal axis buffer': 8, 'Vertical end buffer': 10, 'Horizontal end buffer': 10,
