@@ -5,13 +5,10 @@ presentation_instances.py
 population = [
     # Diagrams
     {'Name': 'default', 'Drawing type': 'Starr class diagram'},
-    {'Name': 'diagnostic', 'Drawing type': 'Starr class diagram'},
 
     {'Name': 'default', 'Drawing type': 'Shlaer-Mellor class diagram'},
-    {'Name': 'diagnostic', 'Drawing type': 'Shlaer-Mellor class diagram'},
 
     {'Name': 'default', 'Drawing type': 'xUML class diagram'},
-    {'Name': 'diagnostic', 'Drawing type': 'xUML class diagram'},
 
     {'Name': 'default', 'Drawing type': 'xUML state machine diagram'},
 
@@ -24,7 +21,10 @@ population = [
     # Frames
     {'Name': 'default', 'Drawing type': 'OS Engineer large frame'},
     {'Name': 'default', 'Drawing type': 'OS Engineer medium frame'},
+    {'Name': 'default', 'Drawing type': 'OS Engineer small frame'},
     {'Name': 'default', 'Drawing type': 'TRI MBSE large frame'},
+    {'Name': 'default', 'Drawing type': 'TRI MBSE medium frame'},
+    {'Name': 'default', 'Drawing type': 'TRI MBSE small frame'},
 
     # Diagnostic
     {'Name': 'default', 'Drawing type': 'Grid Diagnostic'},
