@@ -3,7 +3,7 @@ styledb.py - Loads styles from the flatland database common to all Presentations
 """
 import logging
 from flatland.database.flatlanddb import FlatlandDB as fdb
-from sqlalchemy import select
+from sqlalchemy import select, and_
 from collections import namedtuple
 
 Float_RGB = namedtuple('Float_RGB', 'R G B')
@@ -11,6 +11,21 @@ Line_Style = namedtuple('Line_Style', 'pattern width color')
 Text_Style = namedtuple('Text_Style', 'typeface size slant weight color spacing')
 Dash_Pattern = namedtuple('Dash_Pattern', 'solid blank')
 
+def report_colors(rebuild):
+    fdb(rebuild=rebuild)
+    colors = fdb.MetaData.tables['Color']
+    p = [colors.c.Name]
+    r = and_(
+        (colors.c.Purpose == 'canvas')
+    )
+    q = select(p).where(r)
+    f = fdb.Connection.execute(q).fetchall()
+    print("Canvas colors:")
+    print("---")
+    for i in f:
+        print(i.Name)
+    print("===")
+
 def load_colors():
     colors = fdb.MetaData.tables['Color']
     q = select([colors])
@@ -18,6 +33,13 @@ def load_colors():
     for i in f:
         StyleDB.rgbF[i.Name] = Float_RGB(R=round(i.R / 255, 2), G=round(i.G / 255, 2), B=round(i.B / 255, 2))
 
+def load_color_usages():
+    usages = fdb.MetaData.tables['Color Usage']
+    q = select([usages])
+    f = fdb.Connection.execute(q).fetchall()
+    for i in f:
+        StyleDB.color_usage[i.Name] = i.Color
+
 def load_dash_patterns():
     patterns = fdb.MetaData.tables['Dash Pattern']
     q = select([patterns])
@@ -62,19 +84,25 @@ class StyleDB:
     line_style = {}
     typeface = {}
     text_style = {}
+    color_usage = {}
 
-    def __init__(self):
+    def __init__(self, print_colors=False, rebuild=False):
         """
         Constructor
 
-        :param drawing_type: The Presentation's Drawing Type, 'xUML Class Diagram', for example
-        :param presentation: The Presentation's name, 'default' or 'diagnostic', as examples
+        :param print_colors: True if the user just wants a list of available canvas/usage colors
+        :param rebuild: True if the user requests a database rebuild before reporting colors
         """
         self.logger = logging.getLogger(__name__)
         self.logger.info("Loading common styles from Flatland db")
         # Load all common graphical and text styles in the database (regardless of what we might actually use)
-        load_colors()
-        load_dash_patterns()
-        load_line_styles()
-        load_typefaces()
-        load_text_styles()
\ No newline at end of file
+        if print_colors:
+            # The user wants a list of avaialable colors
+            report_colors(rebuild)
+        else:
+            load_colors()
+            load_color_usages()
+            load_dash_patterns()
+            load_line_styles()
+            load_typefaces()
+            load_text_styles()
\ No newline at end of file
