@@ -47,6 +47,10 @@ class LayoutVisitor(PTNodeVisitor):
         """Keyword argument"""
         return children[0]
 
+    def visit_color(self, node, children):
+        """Keyword argument"""
+        return children[0]
+
     def visit_tpad(self, node, children):
         return {'top': children[0]}
 
@@ -91,8 +95,7 @@ class LayoutVisitor(PTNodeVisitor):
     # Face attachment
     def visit_node_ref(self, node, children):
         """name number?"""
-        return children
-
+        return children[0] if len(children) < 2 else f"{children[0]}_{children[1]}"
 
     def visit_face(self, node, children):
         """Face character"""
@@ -144,9 +147,24 @@ class LayoutVisitor(PTNodeVisitor):
         return path  # { path: { lane: <lane_num>, rut: <rut_displacement> }
 
     # Node
-    def visit_node_expansion(self, node, children):
+    def visit_node_width_expansion(self, node, children):
         """percent expansion"""
-        return {node.rule_name: children[0]}
+        # Convert percentage to ratio ensuring ratio is positive
+        user_percent = children[0]
+        ratio = 0 if user_percent < 0 else round(user_percent / 100, 2)
+        return {node.rule_name: ratio}
+
+    def visit_comp_height_expansion(self, node, children):
+        """percent expansion"""
+        # Convert percentage to ratio ensuring ratio is positive
+        user_percent = children[1]
+        ratio = 0 if user_percent < 0 else round(user_percent / 100, 2)
+        return [children[0], ratio]
+
+    def visit_node_height_expansion(self, node, children):
+        """percent expansion"""
+        d = {k: v for k, v in children}
+        return {node.rule_name: d}
 
     def visit_node_loc(self, node, children):
         """row_span col_span"""
@@ -169,6 +187,10 @@ class LayoutVisitor(PTNodeVisitor):
         """grid_place+"""
         return {'placements': children}
 
+    def visit_color_tag(self, node, children):
+        """tags"""
+        return {node.rule_name: children[0]}
+
     def visit_node_spec(self, node, children):
         """node_name wrap? node_placement"""
         ditems = {k: v for c in children for k, v in c.items()}
@@ -178,6 +200,11 @@ class LayoutVisitor(PTNodeVisitor):
         """All node placements"""
         return children
 
+    # Unary connector
+    def visit_unary_layout(self, node, children):
+        """Unary layout which is just a single stem"""
+        return {'ustem': children[0]}
+
     # Binary connector
     def visit_tertiary_node(self, node, children):
         """Tertiary node face and anchor"""
@@ -191,7 +218,9 @@ class LayoutVisitor(PTNodeVisitor):
 
     def visit_bend(self, node, children):
         """Number of bend where cname appears"""
-        return {node.rule_name: int(children[0])}
+        # return {node.rule_name: int(children[0])}
+        bend = int(children[0])
+        return bend
 
     def visit_tstem(self, node, children):
         """T stem layout info"""
@@ -240,8 +269,8 @@ class LayoutVisitor(PTNodeVisitor):
             raise ConflictingGraftFloat(stem=lface['name'])
         lface['graft'] = graft
         node_ref = lface.pop('node_ref')
-        name = node_ref[0] if len(node_ref) == 1 else f"{node_ref[0]}_{node_ref[1]}"
-        return { name: lface }  # Single element dictionary indexed by the node name
+        # name = node_ref[0] if len(node_ref) == 1 else f"{node_ref[0]}_{node_ref[1]}"
+        return { node_ref: lface }  # Single element dictionary indexed by the node name
 
     def visit_leaf_faces(self, node, children):
         """Combine into dictionary of each leaf face indexed by node name"""
@@ -295,7 +324,16 @@ class LayoutVisitor(PTNodeVisitor):
     # Connector
     def visit_cname_place(self, node, children):
         """Name of connector and the side of the connector axis where it is placed"""
-        cplace = {'cname': children.results['name'][0], 'dir': children.results.get('dir', [1])[0]}
+        # If a value is supplied it will be a single time list, so extract with [0]
+        # If no value is supplied for an optional item, default must also be a single item list [default_value]
+        w = children.results.get('wrap')
+        wrap_value = 1 if not w else w[0]['wrap']
+        cplace = {'cname': children.results['name'][0],  # Required component
+                  'dir': children.results.get('dir', [1])[0],  # many optional components with default values
+                  'bend': children.results.get('bend', [1])[0],
+                  'notch': children.results.get('notch', [0])[0],
+                  'wrap': wrap_value,
+                  }
         return cplace
 
     def visit_connector_layout(self, node, children):
