@@ -16,7 +16,8 @@ from flatland.node_subsystem.single_cell_node import SingleCellNode
 from flatland.node_subsystem.spanning_node import SpanningNode
 from flatland.connector_subsystem.tree_connector import TreeConnector
 from flatland.datatypes.geometry_types import Alignment, VertAlign, HorizAlign
-from flatland.datatypes.command_interface import New_Stem, New_Path, New_Trunk_Branch, New_Offshoot_Branch, New_Branch_Set
+from flatland.datatypes.command_interface import New_Stem, New_Path,\
+     New_Trunk_Branch, New_Offshoot_Branch, New_Branch_Set, New_Compartment
 from typing import Optional, Dict
 from collections import namedtuple
 from flatland.connector_subsystem.straight_binary_connector import StraightBinaryConnector
@@ -26,19 +27,10 @@ from flatland.text.text_block import TextBlock
 
 BranchLeaves = namedtuple('BranchLeaves', 'leaf_stems local_graft next_graft floating_leaf_stem')
 
-def make_node_ref(node_ref) -> str:
-    """
-    Composes a fully qualified placement name based on a node reference in a layout file
-
-    :param node_ref:  A layout reference to a node placement
-    :return: The model elment name if it is only placed once, otherwise name suffixed by an underscore and a numbered placement
-    """
-    return node_ref[0] if len(node_ref) < 2 else f"{node_ref[0]}_{node_ref[1]}"
-
 class XumlClassDiagram:
 
     def __init__(self, xuml_model_path: Path, flatland_layout_path: Path, diagram_file_path: Path,
-                 rebuild: bool, show_grid: bool, nodes_only: bool):
+                 rebuild: bool, show_grid: bool, nodes_only: bool, no_color: bool):
         """Constructor"""
         self.logger = logging.getLogger(__name__)
         self.xuml_model_path = xuml_model_path
@@ -46,6 +38,7 @@ class XumlClassDiagram:
         self.diagram_file_path = diagram_file_path
         self.rebuild = rebuild
         self.show_grid = show_grid
+        self.no_color = no_color
 
         self.logger.info("Parsing the model")
         # Parse the model
@@ -93,12 +86,12 @@ class XumlClassDiagram:
 
         # If there are any relationships, draw them
         if self.subsys.rels and not nodes_only:
-            cp = self.layout.connector_placement
+            cp = {p['cname']: p for p in self.layout.connector_placement}
             for r in self.subsys.rels:  # r is the model data without any layout info
                 rnum = r['rnum']
                 rlayout = cp.get(rnum)  # How this r is to be laid out on the diagram
                 if not rlayout:
-                    self.logger.warning(f"Relationship {rnum} skipped, no placement in layout sheet.")
+                    self.logger.warning(f"Relationship {rnum} skipped, no cplace in layout sheet.")
                     continue
 
                 if 'superclass' in r.keys():
@@ -128,6 +121,8 @@ class XumlClassDiagram:
             diagram_padding=lspec.padding,
             drawoutput=self.diagram_file_path,
             show_grid=self.show_grid,
+            no_color=self.no_color,
+            color=lspec.color,
         )
 
     def draw_classes(self) -> Dict[str, SingleCellNode]:
@@ -145,7 +140,7 @@ class XumlClassDiagram:
             # Get the layout data for this class
             nlayout = np.get(cname)
             if not nlayout:
-                self.logger.warning(f"Skipping class [{cname}] -- No placement specified in layout sheet")
+                self.logger.warning(f"Skipping class [{cname}] -- No cplace specified in layout sheet")
                 continue
 
             # Layout data for all placements
@@ -168,29 +163,34 @@ class XumlClassDiagram:
             # Now assemble all the text content for each class compartment
             # One list item per compartment in descending vertical order of display
             # (class name, attributes and optional methods)
-            text_content = [name_block.text, c['attributes'] + internal_ref ]
+            h_expand = nlayout.get('node_height_expansion', {})
+            text_content = [
+                New_Compartment(content=name_block.text, expansion=h_expand.get(1, 0)),
+                New_Compartment(content=c['attributes'] + internal_ref, expansion=h_expand.get(2, 0)),
+            ]
             if c.get('methods'):
-                text_content.append(c['methods'])
+                text_content.append(
+                    New_Compartment(content=c['methods'], expansion=h_expand.get(1, 0)),
+                )
 
             # The same class may be placed more than once so that the connectors
             # have less bends and crossovers. This is usually, but not limited to,
-            # the placement of imported classes. Since we generate the diagram
+            # the cplace of imported classes. Since we generate the diagram
             # from a single model, there is no harm in duplicating the same class on a
             # diagram.
 
             for i, p in enumerate(nlayout['placements']):
                 h = HorizAlign[p.get('halign', 'CENTER')]
                 v = VertAlign[p.get('valign', 'CENTER')]
-                expansion_percent = max(min(100, nlayout.get('node_expansion', 0)), 1)
-                expansion_ratio = round(expansion_percent / 100, 2)
+                w_expand = nlayout.get('node_width_expansion', 0)
                 # If this is an imported class, append the import reference to the attribute list
                 row_span, col_span = p['node_loc']
                 # If methods were supplied, include them in content
                 # text content includes text for all compartments other than the title compartment
-                # When drawing connectors, we want to attach to a specific node placement
+                # When drawing connectors, we want to attach to a specific node cplace
                 # In most cases, this will just be the one and only indicated by the node name
                 # But if a node is duplicated, i will not be 0 and we add a suffix to the node
-                # name for the additional placement
+                # name for the additional cplace
                 node_name = cname if i == 0 else f'{cname}_{i+1}'
                 if len(row_span) == 1 and len(col_span) == 1:
                     nodes[node_name] = SingleCellNode(
@@ -198,8 +198,9 @@ class XumlClassDiagram:
                         content=text_content,
                         grid=self.flatland_canvas.Diagram.Grid,
                         row=row_span[0], column=col_span[0],
+                        tag=nlayout.get('color_tag', None),
                         local_alignment=Alignment(vertical=v, horizontal=h),
-                        expansion=expansion_ratio
+                        expansion=w_expand,
                     )
                 else:
                     # Span might be only 1 column or row
@@ -213,8 +214,9 @@ class XumlClassDiagram:
                         grid=self.flatland_canvas.Diagram.Grid,
                         low_row=low_row, high_row=high_row,
                         left_column=left_col, right_column=right_col,
+                        tag=nlayout.get('color_tag', None),
                         local_alignment=Alignment(vertical=v, horizontal=h),
-                        expansion=expansion_ratio
+                        expansion=w_expand,
                     )
         return nodes
         # TODO:  Add support for axis offset on stem names
@@ -224,24 +226,24 @@ class XumlClassDiagram:
         # Straight or bent connector?
         tstem = binary_layout['tstem']
         pstem = binary_layout['pstem']
-        reversed = False  # Assume that layout sheet and model order matches
+        _reversed = False  # Assume that layout sheet and model order matches
         astem = binary_layout.get('tertiary_node', None)
 
         t_side = association['t_side']
-        if tstem['node_ref'][0] != t_side['cname']:
+        if tstem['node_ref'] != t_side['cname']:
             # The user put the tstems in the wrong order in the layout file
             # Swap them
             # The node_ref is a list and the first element refers to the model class name
-            # (the 2nd element indicates duplicate placement, if any, and is not relevant for the comparison above)
+            # (the 2nd element indicates duplicate cplace, if any, and is not relevant for the comparison above)
             tstem, pstem = pstem, tstem
-            reversed = True
+            _reversed = True
             self.logger.info(f"Stems order in layout file does not match model, swapping stem order for connector {rnum}")
 
         t_phrase = StemName(
             text=TextBlock(t_side['phrase'], wrap=tstem['wrap']),
             side=tstem['stem_dir'], axis_offset=None, end_offset=None
         )
-        node_ref = make_node_ref(tstem['node_ref'])
+        node_ref = tstem['node_ref']
         t_stem = New_Stem(stem_type='class mult', semantic=t_side['mult'] + ' mult',
                           node=self.nodes[node_ref], face=tstem['face'],
                           anchor=tstem.get('anchor', None), stem_name=t_phrase)
@@ -252,39 +254,42 @@ class XumlClassDiagram:
             text=TextBlock(p_side['phrase'], wrap=pstem['wrap']),
             side=pstem['stem_dir'], axis_offset=None, end_offset=None
         )
-        node_ref = make_node_ref(pstem['node_ref'])
+        node_ref = pstem['node_ref']
         try:
             pnode = self.nodes[node_ref]
         except KeyError:
-            missing_side = "p-stem" if not reversed else "t-stem"
+            missing_side = "p-stem" if not _reversed else "t-stem"
             self.logger.error(f"In layout sheet {missing_side} of {rnum} class [{node_ref}] is not defined in model")
-            sys.exit()
+            sys.exit(1)
         p_stem = New_Stem(stem_type='class mult', semantic=p_side['mult'] + ' mult',
                           node=pnode, face=pstem['face'],
                           anchor=pstem.get('anchor', None), stem_name=p_phrase)
         # There is an optional stem for an association class
         if astem:
-            node_ref = make_node_ref(astem['node_ref'])
+            node_ref = astem['node_ref']
             try:
                 semantic = association['assoc_mult'] + ' mult'
             except KeyError:
                 self.logger.error(
                     f"Layout sheet calls for ternary stem, but class model does not specify any"
                     f" association class on association: {rnum}")
-                sys.exit()
+                sys.exit(1)
             try:
-                node=self.nodes[node_ref]
+                node= self.nodes[node_ref]
             except KeyError:
                 self.logger.error(
                     f"Association class [{node_ref}] is missing in relationship {rnum}"
                 )
-                sys.exit()
+                sys.exit(1)
             a_stem = New_Stem(stem_type='associative mult', semantic=semantic,
                               node=self.nodes[node_ref], face=astem['face'], anchor=astem.get('anchor', None),
                               stem_name=None)
         else:
             a_stem = None
-        rnum_data = ConnectorName(text=rnum, side=binary_layout['dir'], bend=binary_layout['bend'])
+        rnum_data = ConnectorName(
+            text=rnum, side=binary_layout['dir'], bend=binary_layout['bend'], notch=binary_layout['notch'],
+            wrap=1
+        )
 
         paths = None if not binary_layout.get('paths', None) else \
             [New_Path(lane=p['lane'], rut=p['rut']) for p in binary_layout['paths']]
@@ -364,7 +369,7 @@ class XumlClassDiagram:
         One of the rare times it is a good idea to draw one – LS
         """
         trunk_layout = tree_layout['trunk_face']
-        node_ref = make_node_ref(trunk_layout['node_ref'])
+        node_ref = trunk_layout['node_ref']
         trunk_node = self.nodes[node_ref]
 
         # Process trunk branch
@@ -401,6 +406,6 @@ class XumlClassDiagram:
 
         # Now draw the generalization
         branches = New_Branch_Set(trunk_branch=trunk_branch, offshoot_branches=obranches)
-        rnum_data = ConnectorName(text=rnum, side=tree_layout['dir'], bend=None)
+        rnum_data = ConnectorName(text=rnum, side=tree_layout['dir'], bend=None, notch=tree_layout['notch'], wrap=1)
         TreeConnector(diagram=self.flatland_canvas.Diagram, connector_type='generalization',
                       branches=branches, name=rnum_data)
